@@ -2,7 +2,7 @@
 from importlib import metadata
 from importlib.metadata import version
 
-from .console_script import generate_template, sort_excel_file, update_jira_access_token
+from .console_script import generate_template, sort_excel_file, update_jira_info
 from .excel_definition import ExcelDefinition, ExcelDefinitionColumn
 from .excel_operation import (
     output_to_csv_file,
@@ -38,7 +38,7 @@ __all__ = [
     "sort_stories_by_raise_ranking",
     "sort_excel_file",
     "generate_template",
-    "update_jira_access_token",
+    "update_jira_info",
 ]
 
 del metadata
