@@ -23,22 +23,73 @@ RaiseRankingLevelScopeIndexValidationRule = compile(
 # Load -> Parse -> Validate?
 
 
+class PreProcessStep(TypedDict):
+    name: str
+    enabled: bool
+    config: dict
+
+
+def parse_json_item_to_pre_process_step(json_item: Any) -> PreProcessStep:
+    pre_process_step_name = ""
+    pre_process_step_enabled = False
+    pre_process_step_config = {}
+
+    for key, value in json_item.items():
+        if key.lower() == "name".lower():
+            if type(value) is str:
+                pre_process_step_name = value
+            else:
+                raise TypeError("The Name property type should be string.")
+        if key.lower() == "enabled".lower():
+            if type(value) is bool:
+                pre_process_step_enabled = value
+            else:
+                raise TypeError("The Enabled property type should be boolean.")
+        if key.lower() == "config".lower():
+            pre_process_step_config = {}
+            if value is not None and type(value) is dict:
+                for name in value:
+                    # More config support.
+                    if name.lower() == "JiraStatuses".lower():
+                        pre_process_step_config["JiraStatuses"] = value.get(name, None)
+
+    return PreProcessStep(
+        name=pre_process_step_name,
+        enabled=pre_process_step_enabled,
+        config=pre_process_step_config,
+    )
+
+
 class SortStrategy(TypedDict):
-    name: str | NoneType
+    name: str
     priority: int | NoneType
+    enabled: bool
     config: dict
 
 
 def parse_json_item_to_sort_strategy(json_item: Any) -> SortStrategy:
-    strategy_name = None
+    strategy_name = ""
     strategy_priority = 0
+    strategy_enabled = False
     strategy_config = {}
 
     for key, value in json_item.items():
         if key.lower() == "name".lower():
-            strategy_name = value
+            if type(value) is str:
+                strategy_name = value
+            else:
+                raise TypeError("The Name property type should be string.")
         if key.lower() == "priority".lower():
-            strategy_priority = value
+            if type(value) is int:
+                strategy_priority = value
+            else:
+                raise TypeError("The Priority property type should be integer.")
+        if key.lower() == "enabled".lower():
+            # TODO: Some basic error should be through now???
+            if type(value) is bool:
+                strategy_enabled = value
+            else:
+                raise TypeError("The Enabled property type should be boolean.")
         if key.lower() == "config".lower():
             strategy_config = {}
             if value is not None and type(value) is dict:
@@ -48,11 +99,14 @@ def parse_json_item_to_sort_strategy(json_item: Any) -> SortStrategy:
                         strategy_config[
                             name
                         ] = ExcelDefinition.parse_raise_ranking_level_scope_index_expression(
-                            value.get(name)
+                            value.get(name, None)
                         )
 
     return SortStrategy(
-        name=strategy_name, priority=strategy_priority, config=strategy_config
+        name=strategy_name,
+        priority=strategy_priority,
+        enabled=strategy_enabled,
+        config=strategy_config,
     )
 
 
@@ -126,6 +180,7 @@ class ExcelDefinition:
     def __init__(self) -> None:
         self.columns: list[ExcelDefinitionColumn] = []
         self.sort_strategies: list[SortStrategy] = []
+        self.pre_process_steps: list[PreProcessStep] = []
 
     def load(self, content: str) -> "ExcelDefinition":
         """
@@ -141,11 +196,24 @@ class ExcelDefinition:
         try:
             raw_data = json.loads(content)
 
-            for item in raw_data[0]["SortStrategies"]:
-                self.sort_strategies.append(parse_json_item_to_sort_strategy(item))
+            if len(raw_data) > 0:
+                if raw_data[0].get("PreProcessSteps", None) is not None:
+                    for item in raw_data[0]["PreProcessSteps"]:
+                        self.pre_process_steps.append(
+                            parse_json_item_to_pre_process_step(item)
+                        )
 
-            for item in raw_data[1]["Columns"]:
-                self.columns.append(parse_json_item_to_excel_definition_column(item))
+                if raw_data[0].get("SortStrategies", None) is not None:
+                    for item in raw_data[0]["SortStrategies"]:
+                        self.sort_strategies.append(
+                            parse_json_item_to_sort_strategy(item)
+                        )
+
+            if len(raw_data) >= 1:
+                for item in raw_data[1]["Columns"]:
+                    self.columns.append(
+                        parse_json_item_to_excel_definition_column(item)
+                    )
         except Exception:
             raise ValueError(
                 "The JSON structure of the excel definition file is wrong. Please check the documentation: https://github.com/SharryXu/jira-tool"
@@ -155,9 +223,9 @@ class ExcelDefinition:
 
     @staticmethod
     def parse_raise_ranking_level_scope_index_expression(
-        expression: "Any",
+        expression: "Any | None",
     ) -> "set[int] | None":
-        if type(expression) is not str:
+        if expression is None or type(expression) is not str:
             return None
         elif len(expression) == 0 or expression.isspace():
             return set()
@@ -203,6 +271,40 @@ class ExcelDefinition:
     def validate(self) -> "list":
         invalid_definitions = []
 
+        # Validate PreProcessSteps
+        for pre_process_step in self.pre_process_steps:
+            if (
+                pre_process_step["name"] is None
+                or type(pre_process_step["name"]) is not str
+                or pre_process_step["name"].isspace()
+                or len(pre_process_step["name"]) == 0
+            ):
+                invalid_definitions.append(f"The PreProcessStep name is invalid.")
+                # If strategy name is invalid, no need to check more.
+                continue
+
+            if "JiraStatuses".lower() in [
+                config_name.lower() for config_name in pre_process_step["config"].keys()
+            ] and (
+                pre_process_step["name"].lower()
+                != "FilterOutStoryBasedOnJiraStatus".lower()
+            ):
+                invalid_definitions.append(
+                    f"Only FilterOutStoryBasedOnJiraStatus step support JiraStatuses config. PreProcessStep: {pre_process_step['name']}."
+                )
+
+            if (
+                "JiraStatuses".lower()
+                in [
+                    config_name.lower()
+                    for config_name in pre_process_step["config"].keys()
+                ]
+                and type(pre_process_step["config"]["JiraStatuses"]) is not list
+            ):
+                invalid_definitions.append(
+                    f"The format of the Jira Statuses is invalid. PreProcessStep: {pre_process_step['name']}. Supported format like: ['CLOSED', 'PENDING RELEASE']."
+                )
+
         # Validate Strategies
         strategy_priorities: list[int] = []
         for strategy in self.sort_strategies:
@@ -213,10 +315,13 @@ class ExcelDefinition:
                 or len(strategy["name"]) == 0
             ):
                 invalid_definitions.append(f"The strategy name is invalid.")
+                # If strategy name is invalid, no need to check more.
+                continue
+
             if (
                 strategy["priority"] is None
                 or type(strategy["priority"]) is not int
-                or strategy["priority"] <= 0
+                or strategy["priority"] < 0
             ):
                 invalid_definitions.append(
                     f"The strategy priority is invalid. Strategy: {strategy['name']}"
@@ -224,8 +329,19 @@ class ExcelDefinition:
             else:
                 strategy_priorities.append(strategy["priority"])
 
+            if "ParentScopeIndexRange".lower() in [
+                config_name.lower() for config_name in strategy["config"].keys()
+            ] and (
+                strategy["name"].lower() != "SortOrder".lower()
+                and strategy["name"].lower() != "RaiseRanking".lower()
+            ):
+                invalid_definitions.append(
+                    f"Only RaiseRanking and SortOrder strategy support ParentScopeIndexRange config. Strategy: {strategy['name']}."
+                )
+
             if (
-                "ParentScopeIndexRange" in strategy["config"]
+                "ParentScopeIndexRange".lower()
+                in [config_name.lower() for config_name in strategy["config"].keys()]
                 and type(strategy["config"]["ParentScopeIndexRange"]) is not set
             ):
                 invalid_definitions.append(
@@ -238,6 +354,7 @@ class ExcelDefinition:
             )
 
         # Validate the Columns
+        exist_story_id = False
         exist_indexes = []
         exist_inline_weights = []
         for column in self.get_columns():
@@ -259,6 +376,10 @@ class ExcelDefinition:
                     f"Column name cannot be empty. Index: {column_index}"
                 )
                 continue
+
+            if column_name.lower() == "StoryId".lower():
+                exist_story_id = True
+
             # Check Missing/Duplicate Index
             if type(column_index) is not int:
                 invalid_definitions.append(
@@ -365,6 +486,11 @@ class ExcelDefinition:
                         f"Jira Field Mapping has the invalid path. Column: {column_name}"
                     )
 
+        if len(self.columns) > 0 and exist_story_id is False:
+            invalid_definitions.append(
+                "Must have a column named StoryId so that program can identify the record."
+            )
+
         if len(invalid_definitions) == 0:
             self.columns.sort(key=lambda c: c["index"], reverse=False)
 
@@ -413,9 +539,20 @@ class ExcelDefinition:
     def max_column_index(self) -> int:
         return self.columns[len(self.columns) - 1]["index"]
 
-    def get_sort_strategies(self) -> "list[SortStrategy]":
-        self.sort_strategies.sort(key=_sort_priority_map, reverse=False)
-        return self.sort_strategies
+    def get_sort_strategies(self, enabled: bool = True) -> "list[SortStrategy]":
+        result: list[SortStrategy] = []
+        for sort_strategy in self.sort_strategies:
+            if sort_strategy["enabled"] == enabled:
+                result.append(deepcopy(sort_strategy))
+        result.sort(key=_sort_priority_map, reverse=False)
+        return result
+
+    def get_pre_process_steps(self, enabled: bool = True) -> "list[PreProcessStep]":
+        result: list[PreProcessStep] = []
+        for pre_process_step in self.pre_process_steps:
+            if pre_process_step["enabled"] == enabled:
+                result.append(deepcopy(pre_process_step))
+        return result
 
     def total_count(self):
         return len(self.columns)
