@@ -16,7 +16,6 @@ from urllib3 import disable_warnings
 
 from .excel_definition import *
 from .jira_client import *
-from .jira_configuration import *
 from .milestone import *
 from .priority import *
 from .sprint_schedule import *
@@ -228,12 +227,93 @@ def output_to_excel_file(
     wb.close()
 
 
+def _query_jira_information(stories: list[Story], excel_definition: ExcelDefinition):
+    from dotenv import load_dotenv
+
+    load_dotenv(ASSETS / ".env")
+
+    jira_url: str | None = os.environ.get("JIRA_URL", default=None)
+    if (
+        # TODO: Avoid duplicate logic.
+        jira_url is None
+        or jira_url.isspace()
+        or len(jira_url) == 0
+    ):
+        print(
+            "The jira url is invalid. Please use the update-jira-info command to add/update url."
+        )
+        return
+
+    jira_acccess_token: str | None = os.environ.get("JIRA_ACCESS_TOKEN", default=None)
+    if (
+        jira_acccess_token is None
+        or jira_acccess_token.isspace()
+        or len(jira_acccess_token) == 0
+    ):
+        print(
+            "The jira access token is invalid. Please use the update-jira-info command to add/update token."
+        )
+        return
+
+    jira_client = JiraClient(jira_url, jira_acccess_token)
+
+    if not jira_client.health_check():
+        print(
+            "The jira access token is revoked. Please use the update-jira-info command to add/update token."
+        )
+        return
+
+    jira_fields = []
+
+    for definition_column in excel_definition.get_columns():
+        if (
+            definition_column["name"] is None
+            or definition_column["jira_field_mapping"] is None
+        ):
+            continue
+        else:
+            jira_fields.append(
+                {
+                    "name": definition_column["name"],
+                    "jira_name": definition_column["jira_field_mapping"]["name"],
+                    "jira_path": definition_column["jira_field_mapping"]["path"],
+                }
+            )
+
+    jira_query_result = jira_client.get_stories_detail(
+        [story["storyId"].strip() for story in stories], jira_fields
+    )
+
+    for story in stories:
+        story_id: str = story["storyId"].lower().strip()
+        if story_id in jira_query_result:
+            for jira_field in jira_fields:
+                story[jira_field["name"]] = jira_query_result[story_id][
+                    jira_field["jira_name"]
+                ]
+        else:
+            # Story ID has been changed because of convertion.
+            temp_result = jira_client.get_stories_detail([story_id], jira_fields)
+            if len(temp_result) > 0:
+                story["storyId"] = list(temp_result.keys())[0].upper()
+                for jira_field in jira_fields:
+                    story[jira_field["name"]] = list(temp_result.values())[0][
+                        jira_field["jira_name"]
+                    ].upper()
+                print(
+                    f"Story id has been changed. Previous: {story_id.upper()}, Current: {story['storyId'].upper()}"
+                )
+            else:
+                print(f"Cannot find related information for story: {story_id}")
+                story.need_sort = False  # TODO: Avoid NoneType compare issue.
+                continue
+
+
 def process_excel_file(
     input_file: "str | Path",
     output_file: "str | Path",
     excel_definition_file: "str | Path | None" = None,
     sprint_schedule_file: "str | Path | None" = None,
-    jira_configuration_file: "str | Path | None" = None,
     over_write: bool = True,
 ):
     """
@@ -293,118 +373,56 @@ def process_excel_file(
         print("There are no stories inside the excel file.")
         return
 
-    # TODO: Get Jira Status and filter stories.
-    from dotenv import load_dotenv
+    # Execute pre-process steps
+    pre_process_steps = excel_definition.get_pre_process_steps()
 
-    load_dotenv(ASSETS / ".env")
+    for pre_process_step in pre_process_steps:
+        print(f"Executing step: {pre_process_step['name']}...")
+        if pre_process_step["name"].lower() in "RetrieveJiraInformation".lower():
+            need_call_jira_api: bool = False
+            for excel_definition_column in excel_definition.get_columns():
+                if excel_definition_column["jira_field_mapping"] is not None:
+                    need_call_jira_api = True
+                    break
 
-    jira_url: str | None = os.environ.get("JIRA_URL", default=None)
-    if (
-        # TODO: Avoid duplicate logic.
-        jira_url is None
-        or jira_url.isspace()
-        or len(jira_url) == 0
-    ):
-        print(
-            "The jira url is invalid. Please use the update-jira-info command to add/update url."
-        )
-        return
-
-    jira_acccess_token: str | None = os.environ.get("JIRA_ACCESS_TOKEN", default=None)
-    if (
-        # TODO: Avoid duplicate logic.
-        jira_acccess_token is None
-        or jira_acccess_token.isspace()
-        or len(jira_acccess_token) == 0
-    ):
-        print(
-            "The jira access token is invalid. Please use the update-jira-info command to add/update token."
-        )
-        return
-
-    jira_client = JiraClient(jira_url, jira_acccess_token)
-
-    if not jira_client.health_check():
-        print(
-            "The jira access token is revoked. Please use the update-jira-info command to add/update token."
-        )
-        return
-
-    noneed_sort_statuses = get_no_need_sort_statuses(jira_configuration_file)
-
-    stories_has_story_id = []
-    for story in stories:
-        if story["storyId"] is not None:
-            stories_has_story_id.append(story["storyId"].strip())
-
-    jira_fields = []
-
-    for definition_column in excel_definition.get_columns():
-        if (
-            definition_column["name"] is None
-            or definition_column["jira_field_mapping"] is None
+            if need_call_jira_api:
+                stories_need_call_jira: list[Story] = []
+                for story in stories:
+                    if story.need_sort:
+                        stories_need_call_jira.append(story)
+                _query_jira_information(stories_need_call_jira, excel_definition)
+        elif pre_process_step["name"].lower() in "FilterOutStoryWithoutId".lower():
+            for story in stories:
+                if story["storyId"] is None:
+                    story.need_sort = False
+        elif (
+            pre_process_step["name"].lower()
+            in "FilterOutStoryBasedOnJiraStatus".lower()
         ):
-            continue
-        else:
-            jira_fields.append(
-                {
-                    "name": definition_column["name"],
-                    "jira_name": definition_column["jira_field_mapping"]["name"],
-                    "jira_path": definition_column["jira_field_mapping"]["path"],
-                }
-            )
+            for story in stories:
+                if story["status"] is not None and story[
+                    "status"
+                ].upper() in pre_process_step["config"].get("JiraStatuses", []):
+                    story.need_sort = False
+        print(f"Executing finish.")
 
-    jira_query_result = jira_client.get_stories_detail(
-        stories_has_story_id, jira_fields
-    )
-
-    print("Quering all stories' jira properties...")
-
-    stories_need_sort = []
     stories_no_need_sort = []
+    stories_need_sort = []
 
     for story in stories:
-        if story["storyId"] is None:
+        if story.need_sort:
             stories_need_sort.append(story)
-            continue
-        story_id: str = story["storyId"].lower().strip()
-        if story_id in jira_query_result:
-            for jira_field in jira_fields:
-                story[jira_field["name"]] = jira_query_result[story_id][
-                    jira_field["jira_name"]
-                ]
         else:
-            # Story ID has been changed because of convertion.
-            temp_result = jira_client.get_stories_detail([story_id], jira_fields)
-            if len(temp_result) > 0:
-                story["storyId"] = list(temp_result.keys())[0].upper()
-                for jira_field in jira_fields:
-                    story[jira_field["name"]] = list(temp_result.values())[0][
-                        jira_field["jira_name"]
-                    ].upper()
-                print(
-                    f"Story id has been changed. Previous: {story_id.upper()}, Current: {story['storyId'].upper()}"
-                )
-            else:
-                print(f"Cannot find related information for story: {story_id}")
-                stories_no_need_sort.append(story)
-                continue
-
-        # TODO: Add filter strategy to the program.
-        if story["status"].upper() in noneed_sort_statuses:
-            story.need_sort = False
             stories_no_need_sort.append(story)
-            continue
-
-        stories_need_sort.append(story)
 
     # Execute sorting logic.
     sort_strategies = excel_definition.get_sort_strategies()
 
     for sort_strategy in sort_strategies:
+        print(f"Executing {sort_strategy['name']} sorting...")
         if sort_strategy["name"] is None:
             continue
-        if sort_strategy["name"].lower() in "InlineWeights".lower():
+        elif sort_strategy["name"].lower() in "InlineWeights".lower():
             stories_need_sort = sorted(stories_need_sort, reverse=True)
         elif sort_strategy["name"].lower() in "SortOrder".lower():
             sort_stories_by_property_and_order(
@@ -414,11 +432,11 @@ def process_excel_file(
             stories_need_sort = sort_stories_by_raise_ranking(
                 stories_need_sort, excel_definition, sort_strategy["config"]
             )
-        print(f"Executed {sort_strategy['name']} sorting")
+        print(f"Executing finish.")
 
     output_to_excel_file(
         output_file,
-        stories_need_sort + stories_no_need_sort,
+        stories_need_sort + stories_no_need_sort,  # First output the sorted stories.
         excel_definition,
         excel_columns,
         over_write,
