@@ -13,7 +13,7 @@ from requests.status_codes import codes
 class JiraClient:
     def __init__(self, url, access_token: str) -> None:
         self.base_url: str = url
-        self.timeout: int = 3
+        self.timeout: int = 20
         self.access_token: str = access_token
         self.headers = {
             "Accept": "application/json",
@@ -78,7 +78,7 @@ class JiraClient:
                                 field_value = ""
                                 break
                             field_value = field_value.get(field_path, None)
-                        fields_result[field["jira_name"]] = field_value
+                        fields_result[field_name] = field_value
                     final_result[issue["key"].lower()] = fields_result
 
                 return final_result
