@@ -17,7 +17,7 @@ class SprintScheduleStore:
             JSON string content
         """
         if content is None:
-            raise ValueError("The content is invalid.")
+            return
 
         try:
             raw_data = json.loads(content)
@@ -39,7 +39,7 @@ class SprintScheduleStore:
                 priority = 0
         except Exception:
             raise ValueError(
-                "The JSON file structure is invalid. Please check the documentation: https://github.com/SharryXu/jira-tool"
+                "The JSON structure of the sprint schedule file is wrong. Please check the documentation: https://github.com/SharryXu/jira-tool"
             )
 
     def load_file(self, file: "str | Path"):
