@@ -123,7 +123,7 @@ class Story(object):
 
     def calc_sprint_schedule(self, sprint_schedule: SprintScheduleStore):
         for column in self.factory.columns:
-            if column["type"] is Milestone:
+            if column["type"] is Milestone and self[column["name"]] is not None:
                 self[column["name"]].calc_priority(sprint_schedule)
 
     # Currently, comparing story only consider the Priority properties.
@@ -334,13 +334,13 @@ def _internal_sort_stories_by_property_and_order_considering_parent_range(
                 end_index = i
                 break
         # Same parent level process
-        stories[begin_index : end_index + 1]
         for column_name, sort_order in reversed(sort_rules):
-            stories[begin_index : end_index + 1] = sorted(
-                stories[begin_index : end_index + 1],
-                key=attrgetter(column_name),
-                reverse=sort_order,
-            )
+            if begin_index != end_index:
+                stories[begin_index : end_index + 1] = sorted(
+                    stories[begin_index : end_index + 1],
+                    key=attrgetter(column_name),
+                    reverse=sort_order,
+                )
 
         begin_index = end_index + 1
         end_index = begin_index
